@@ -1,16 +1,20 @@
+import atexit
 import base64
 import importlib
+import json
 import logging
 import os
 import platform
 import time
 import traceback
+from collections import OrderedDict
 from functools import partial
 from itertools import product
-from typing import Any, Optional
+from typing import Any, Optional, List
 from pathlib import Path
 import webbrowser
 
+import robot
 from robot.libraries.BuiltIn import BuiltIn, RobotNotRunningError
 from SeleniumLibrary import SeleniumLibrary, EMBED
 from SeleniumLibrary.base import keyword
@@ -22,7 +26,7 @@ from SeleniumLibrary.keywords import (
 )
 from selenium.webdriver import ChromeOptions
 
-from RPA.core import notebook, webdriver
+from RPA.core import webdriver, notebook
 from RPA.core.locators import LocatorsDatabase, BrowserLocator
 
 
@@ -40,7 +44,7 @@ class BrowserNotFoundError(ValueError):
     """Raised when browser can't be initialized."""
 
 
-class Browser(SeleniumLibrary):
+class Selenium(SeleniumLibrary):
     """Browser is a web testing library for Robot Framework,
     based on the popular SeleniumLibrary.
 
@@ -157,6 +161,30 @@ class Browser(SeleniumLibrary):
     | `Click Element` | //div[@id="foo"]//h1 |
     | `Click Element` | (//div)[2]           |
 
+    === Chaining locators ===
+
+    It's possible to chain multiple locators together as a single locator. Each chained locator must start
+    with a locator strategy. Chained locators must be separated with a single space, two greater than characters,
+    and followed with a space. It's also possible to mix different locator strategies, such as css or xpath.
+    Also, a list can also be used to specify multiple locators, for instance when the chaining separator
+    would conflict with the actual locator, or when an existing web element is used as a base.
+
+    Although all locators support chaining, some locator strategies don't chain properly with previous values.
+    This is because some locator strategies use JavaScript to find elements and JavaScript is executed
+    for the whole browser context and not for the element found by the previous locator. Locator strategies
+    that support chaining are the ones that are based on the Selenium API, such as `xpath` or `css`, but for example
+    chaining is not supported by `sizzle` or `jquery`.
+
+    Examples:
+    | `Click Element` | css:.bar >> xpath://a | # To find a link which is present inside an element with class "bar" |
+
+    List examples:
+    | ${locator_list} =             | `Create List`   | css:div#div_id            | xpath://*[text(), " >> "] |
+    | `Page Should Contain Element` | ${locator_list} |                           |                           |
+    | ${element} =                  | Get WebElement  | xpath://*[text(), " >> "] |                           |
+    | ${locator_list} =             | `Create List`   | css:div#div_id            | ${element}                |
+    | `Page Should Contain Element` | ${locator_list} |                           |                           |
+
     == Using WebElements ==
 
     In addition to specifying a locator as a string, it is possible to use
@@ -324,37 +352,11 @@ class Browser(SeleniumLibrary):
     The run-on-failure functionality can be disabled by using a special value
     ``NOTHING`` or anything considered false (see `Boolean arguments`)
     such as ``NONE``.
-
-    = Boolean arguments =
-
-    Some keywords accept arguments that are handled as Boolean values true or
-    false. If such an argument is given as a string, it is considered false if
-    it is either empty or case-insensitively equal to ``false``, ``no``, ``off``,
-     ``0`` or ``none``. Other strings are considered true regardless of their value and
-    other argument types are tested using the same
-    [https://docs.python.org/3/library/stdtypes.html#truth-value-testing|rules as in Python].
-
-    True examples:
-
-    | `Set Screenshot Directory` | ${RESULTS} | persist=True    | # Strings are generally true.    |
-    | `Set Screenshot Directory` | ${RESULTS} | persist=yes     | # Same as the above.             |
-    | `Set Screenshot Directory` | ${RESULTS} | persist=${TRUE} | # Python True is true.           |
-    | `Set Screenshot Directory` | ${RESULTS} | persist=${42}   | # Numbers other than 0 are true. |
-
-    False examples:
-
-    | `Set Screenshot Directory` | ${RESULTS} | persist=False    | # String false is false.        |
-    | `Set Screenshot Directory` | ${RESULTS} | persist=no       | # Also string no is false.      |
-    | `Set Screenshot Directory` | ${RESULTS} | persist=NONE     | # String NONE is false.         |
-    | `Set Screenshot Directory` | ${RESULTS} | persist=${EMPTY} | # Empty string is false.        |
-    | `Set Screenshot Directory` | ${RESULTS} | persist=${FALSE} | # Python False is false.        |
-    | `Set Screenshot Directory` | ${RESULTS} | persist=${NONE}  | # Python None is false.         |
     """  # noqa: E501
 
     ROBOT_LIBRARY_SCOPE = "GLOBAL"
     ROBOT_LIBRARY_DOC_FORMAT = "ROBOT"
 
-    AUTOMATIC_BROWSER_SELECTION = "AUTO"
     AVAILABLE_OPTIONS = {
         "chrome": "ChromeOptions",
         "firefox": "FirefoxOptions",
@@ -363,7 +365,8 @@ class Browser(SeleniumLibrary):
     }
 
     def __init__(self, *args, **kwargs) -> None:
-        # We need to pop this from kwargs before passing kwargs to SeleniumLibrary
+        # We need to pop our kwargs before passing kwargs to SeleniumLibrary
+        self.auto_close = kwargs.pop("auto_close", True)
         self.locators_path = kwargs.pop("locators_path", None)
 
         # Parse user-given plugins
@@ -385,13 +388,31 @@ class Browser(SeleniumLibrary):
         # Add support for locator aliases
         self._element_finder.register("alias", self._find_by_alias, persist=True)
 
-        self._embedding_screenshots = False
-        self._previous_screenshot_directory = None
         # Embed screenshots in logs by default
         if not notebook.IPYTHON_AVAILABLE:
             self._embedding_screenshots = True
             self._previous_screenshot_directory = self.set_screenshot_directory(EMBED)
+        else:
+            self._embedding_screenshots = False
+            self._previous_screenshot_directory = None
+
         self.download_preferences = {}
+        self._close_on_exit()
+
+    def _close_on_exit(self):
+        """Register function to clean leftover webdrivers on process exit."""
+        connections = self._drivers._connections  # pylint: disable=protected-access
+
+        def stop_drivers():
+            if not self.auto_close:
+                return
+            for driver in connections:
+                try:
+                    driver.service.stop()
+                except Exception:  # pylint: disable=broad-except
+                    pass
+
+        atexit.register(stop_drivers)
 
     @property
     def location(self) -> str:
@@ -415,83 +436,132 @@ class Browser(SeleniumLibrary):
     @keyword
     def open_available_browser(
         self,
-        url: str,
+        url: Optional[str] = None,
         use_profile: bool = False,
-        headless: Optional[bool] = None,
+        headless: Any = "AUTO",
         maximized: bool = False,
-        browser_selection: Any = AUTOMATIC_BROWSER_SELECTION,
+        browser_selection: Any = "AUTO",
         alias: Optional[str] = None,
         profile_name: Optional[str] = None,
         profile_path: Optional[str] = None,
         preferences: Optional[dict] = None,
         proxy: str = None,
+        user_agent: Optional[str] = None,
+        download: Any = "AUTO",
     ) -> int:
         # pylint: disable=C0301
-        """Opens the first available browser in the system in preferred order, or the
-        given browser (``browser_selection``).
+        """Attempts to open a browser on the user's device from a set of
+        supported browsers. Automatically downloads a corresponding webdriver
+        if none is already installed.
 
-        ``url`` URL to open
+        Optionally can be given a ``url`` as the first argument,
+        to open the browser directly to the given page.
 
-        ``use_profile`` Set browser profile, default ``False`` (Chrome/Chromium only)
+        Returns either a generated index or a custom ``alias`` for the
+        browser instance. The returned value can be used to refer to that
+        specific browser instance in other keywords.
 
-        ``headless`` Run in headless mode, default ``False``
+        If the browser should start in a maximized window, this can be
+        enabled with the argument ``maximized``, but is disabled by default.
 
-        ``maximized`` Run window maximized, default ``False``
-
-        ``browser_selection`` browser name, default ``AUTOMATIC_BROWSER_SELECTION``
-
-        ``alias`` Custom name for browser
-
-        ``profile_name`` Name of profile (if profile enabled)
-
-        ``profile_path`` Path to profiles (if profile enabled)
-
-        ``preferences`` Profile preferences (Chrome/Chromium only)
-
-        ``proxy`` Proxy server address (Chrome only)
-
-        Returns an index of the webdriver session.
-
-        === Process of opening a browser ===
-
-        1. Get the order of browsers
-
-        2. Loop the list of preferred browsers
-
-            a. Set the webdriver options for the browser
-
-            b. Create the webdriver using existing installation
-
-            c. (If step b. failed) Download and install webdriver, try again
-
-            d. (If step c. failed) Try starting webdriver in headless mode
-
-        3. Open the URL
-
-        Returns index or custom alias for the browser instance.
-
-        Raises ``BrowserNotFoundError`` if unable to open the browser.
-
-        For information about Safari webdriver setup, see
-        https://developer.apple.com/documentation/webkit/testing_with_webdriver_in_safari
+        For certain applications it might also be required to force a
+        certain user-agent string for Selenium, which can be overriden
+        with the ``user_agent`` argument.
 
         Example:
 
-        | ${idx1} | Open Available Browser | https://www.robocorp.com |
-        | ${idx2} | Open Available Browser | ${URL} | browser_selection=opera,firefox |
-        | Open Available Browser | ${URL} | headless=True | proxy=localhost:8899 |
+        | Open Available Browser | https://www.robocorp.com |
+        | ${index}= | Open Available Browser | ${URL} | browser_selection=opera,firefox |
+        | Open Available Browser | ${URL} | headless=True | alias=HeadlessBrowser |
+
+        == Browser order ==
+
+        The default order of supported browsers is based on the operating system
+        and is as follows:
+
+        | Platform    | Default order                    |
+        | ``Windows`` | Chrome, Firefox, Edge, IE, Opera |
+        | ``Linux``   | Chrome, Firefox, Opera           |
+        | ``Darwin``  | Chrome, Safari, Firefox, Opera   |
+
+        The order can be overriden with a custom list by using the argument
+        ``browser_selection``. The argument can be either a comma-separated
+        string or a list object.
+
+        == Webdriver download ==
+
+        The library can (if requested) automatically download webdrivers
+        for all supported browsers. This can be controlled with the argument
+        ``download``.
+
+        If the value is ``False``, it will only attempt to start
+        webdrivers found from the system PATH.
+
+        If the value is ``True``, it will download a webdriver that matches
+        the current browser.
+
+        By default the argument has the value ``AUTO``, which means it
+        first attempts to use webdrivers found in PATH and if that fails
+        forces a webdriver download.
+
+        == Opening process ==
+
+        1. Parse list of preferred browser order. If not given, use values
+           from above table.
+
+        2. Loop through listed browsers:
+
+            a. Set the webdriver options for the browser.
+
+            b. Download webdriver (if requested).
+
+            c. Attempt to launch the webdriver and stop the loop if successful.
+
+        3. Return index/alias if webdriver was created, or raise an exception
+           if no browsers were successfully opened.
+
+        == Headless mode ==
+
+        If required, the browser can also run `headless`, which means that
+        it does not create a visible window. Generally a headless browser is
+        slightly faster, but might not support all features a normal browser does.
+
+        One typical use-case for headless mode is in cloud containers,
+        where there is no display available. It also prevents manual interaction
+        with the browser, which can be either a benefit or a drawback depending on
+        the context.
+
+        It can be explicitly enabled or disabled with the argument ``headless``.
+        By default it will be disabled, unless it detects that it is running
+        in a Linux environment without a display, i.e. a container.
+
+        == Chrome options ==
+
+        Some features are currently available only for Chrome/Chromium.
+        This includes using an existing user profile. By default Selenium
+        uses a new profile for each session, but it can use an existing
+        one by enabling the ``use_profile`` argument.
+
+        If a custom profile is stored somewhere outside of the default location,
+        the path to the profiles directory and the name of the profile can
+        be controlled with ``profile_path`` and ``profile_name`` respectively.
+
+        Profile preferences can be further overriden with the ``preferences``
+        argument by giving a dictionary of key/value pairs.
+
+        Chrome can additionally connect through a ``proxy``, which
+        should be given as either local or remote address.
         """  # noqa: E501
         # pylint: disable=redefined-argument-from-local
-        browser_options = self._get_browser_order(browser_selection)
-        headless_options = [headless] if headless is not None else [False, True]
-        download_options = [False, True]
+        browsers = self._arg_browser_selection(browser_selection)
+        downloads = self._arg_download(download)
+        headless = self._arg_headless(headless)
 
-        # Try a combination of all options until a browser starts
+        attempts = []
         index_or_alias = None
-        options = []
-        for browser, headless, download in product(
-            browser_options, headless_options, download_options
-        ):
+
+        # Try all browsers in preferred order
+        for browser, download in product(browsers, downloads):
             try:
                 self.logger.debug(
                     "Creating webdriver for '%s' (headless: %s, download: %s)",
@@ -508,11 +578,12 @@ class Browser(SeleniumLibrary):
                     profile_path,
                     preferences,
                     proxy,
+                    user_agent,
                 )
                 index_or_alias = self._create_webdriver(
                     browser, alias, download, **kwargs
                 )
-                options.append((browser, headless, download, ""))
+                attempts.append((browser, download, ""))
                 self.logger.info(
                     "Created %s browser with arguments: %s",
                     browser,
@@ -520,47 +591,66 @@ class Browser(SeleniumLibrary):
                 )
                 break
             except Exception as error:  # pylint: disable=broad-except
-                options.append((browser, headless, download, error))
+                attempts.append((browser, download, error))
                 self.logger.debug(traceback.format_exc())
 
         # Log table of all attempted combinations
+        table_headers = ["Browser", "Download", "Error"]
         try:
-            table = html_table(
-                header=["Browser", "Headless", "Download", "Error"], rows=options
-            )
+            table = html_table(header=table_headers, rows=attempts)
             BuiltIn().log("<p>Attempted combinations:</p>" + table, html=True)
         except RobotNotRunningError:
             pass
 
         # No webdriver was started
         if index_or_alias is None:
-            raise BrowserNotFoundError(
-                "No valid browser found from: {}".format(
-                    ", ".join(browser for browser in browser_options)
-                )
-            )
+            errors = OrderedDict((browser, error) for browser, _, error in attempts)
+
+            msg = "Failed to start a browser:\n"
+            for browser, error in errors.items():
+                msg += f"- {browser}: {error}\n"
+
+            notebook.notebook_table(attempts, columns=table_headers)
+            raise BrowserNotFoundError(msg)
+
+        if url is not None:
+            self.go_to(url)
 
-        self.go_to(url)
         return index_or_alias
 
-    def _get_browser_order(self, browser_selection: Any) -> list:
-        """Get a list of browsers that will be used for open browser
-        keywords. Will be one or many.
-
-        ``browser_selection`` ``AUTOMATIC_BROWSER_SELECTION`` will be OS-specific list,
-            or one named browser, eg. "Chrome"
-        """
-        if browser_selection == self.AUTOMATIC_BROWSER_SELECTION:
-            preferable_browser_order = webdriver.DRIVER_PREFERENCE.get(
+    def _arg_browser_selection(self, browser_selection: Any) -> List:
+        """Parse argument for browser selection."""
+        if str(browser_selection).strip().lower() == "auto":
+            order = webdriver.DRIVER_PREFERENCE.get(
                 platform.system(), webdriver.DRIVER_PREFERENCE["default"]
             )
         else:
-            preferable_browser_order = (
+            order = (
                 browser_selection
                 if isinstance(browser_selection, list)
                 else browser_selection.split(",")
             )
-        return preferable_browser_order
+        return order
+
+    def _arg_download(self, download: Any) -> List:
+        """Parse argument for webdriver download."""
+        if str(download).strip().lower() == "auto":
+            return [False, True]
+        else:
+            return [bool(download)]
+
+    def _arg_headless(self, headless: Any) -> bool:
+        """Parse argument for headless mode."""
+        if str(headless).strip().lower() == "auto":
+            # If in Linux and with no valid display, we can assume we are in a container
+            headless = platform.system() == "Linux" and not (
+                os.environ.get("DISPLAY") or os.environ.get("WAYLAND_DISPLAY")
+            )
+            if headless:
+                self.logger.info("Autodetected headless environment")
+            return headless
+        else:
+            return bool(headless)
 
     def _get_driver_args(
         self,
@@ -572,6 +662,7 @@ class Browser(SeleniumLibrary):
         profile_path: Optional[str] = None,
         preferences: Optional[dict] = None,
         proxy: str = None,
+        user_agent: Optional[str] = None,
     ) -> dict:
         """Get browser and webdriver arguments for given options."""
         preferences = preferences or {}
@@ -592,6 +683,9 @@ class Browser(SeleniumLibrary):
         if maximized:
             options.add_argument("--start-maximized")
 
+        if user_agent:
+            options.add_argument(f"user-agent={user_agent}")
+
         if browser != "chrome":
             kwargs["options"] = options
             if use_profile:
@@ -624,7 +718,7 @@ class Browser(SeleniumLibrary):
                 kwargs["service_log_path"] = "chromedriver.log"
                 kwargs["service_args"] = ["--verbose"]
 
-            kwargs["chrome_options"] = options
+            kwargs["options"] = options
 
         return kwargs, options.arguments
 
@@ -645,6 +739,9 @@ class Browser(SeleniumLibrary):
         options.add_argument("--headless")
         options.add_argument("--disable-gpu")
 
+        if browser.lower() == "chrome":
+            options.add_argument("--window-size=1440,900")
+
     def _set_user_profile(
         self,
         options: dict,
@@ -685,20 +782,47 @@ class Browser(SeleniumLibrary):
         if profile_name is not None:
             options.add_argument(f"--profile-directory={profile_name}")
 
-    def _create_webdriver(self, browser, alias, download, **kwargs):
-        """Create a webdriver instance for the given browser.
+    def _create_webdriver(
+        self, browser: str, alias: Optional[str], download: bool, **kwargs
+    ):
+        """Create a webdriver instance with given options.
 
-        Returns an index/alias of the webdriver session,
-        or ``None`` if a webdriver was not initialized.
+        If webdriver download is requested, try using a cached
+        version first and if that fails force a re-download.
         """
-        executable = webdriver.executable(browser, download)
-        if executable:
-            kwargs.setdefault("executable_path", executable)
+        browser = browser.lower()
 
-        library = BrowserManagementKeywords(self)
-        browser = browser.lower().capitalize()
+        def _create_driver(path=None):
+            options = dict(kwargs)
+            if path is not None:
+                options["executable_path"] = path
 
-        return library.create_webdriver(browser, alias, **kwargs)
+            lib = BrowserManagementKeywords(self)
+            return lib.create_webdriver(browser.capitalize(), alias, **options)
+
+        # No download requested
+        if not download:
+            return _create_driver()
+
+        # Check if webdriver is available for given browser
+        if browser not in webdriver.AVAILABLE_DRIVERS:
+            raise ValueError(f"Webdriver download not available for {browser.title()}")
+
+        # Try to use webdriver already in cache
+        path_cache = webdriver.cache(browser)
+        if path_cache:
+            try:
+                return _create_driver(path_cache)
+            except Exception:  # pylint: disable=broad-except
+                pass
+
+        # Try to download webdriver
+        path_download = webdriver.download(browser)
+        if path_download:
+            return _create_driver(path_download)
+
+        # No webdriver required
+        return _create_driver()
 
     @keyword
     def open_chrome_browser(
@@ -712,6 +836,7 @@ class Browser(SeleniumLibrary):
         profile_path: Optional[str] = None,
         preferences: Optional[dict] = None,
         proxy: str = None,
+        user_agent: Optional[str] = None,
     ) -> int:
         """Open Chrome browser. See ``Open Available Browser`` for
         descriptions of arguments.
@@ -727,6 +852,7 @@ class Browser(SeleniumLibrary):
             profile_path=profile_path,
             preferences=preferences,
             proxy=proxy,
+            user_agent=user_agent,
         )
 
     @keyword
@@ -745,9 +871,7 @@ class Browser(SeleniumLibrary):
         """
         options = ChromeOptions()
         options.add_experimental_option("debuggerAddress", f"localhost:{port:d}")
-        create = partial(
-            self._create_webdriver, "Chrome", alias, chrome_options=options
-        )
+        create = partial(self._create_webdriver, "Chrome", alias, options=options)
 
         try:
             return create(download=False)
@@ -783,12 +907,12 @@ class Browser(SeleniumLibrary):
 
         Example:
 
-        | Screenshot | locator=//img[@alt="Google"] | filename=locator.png |             # element screenshot, defined filename
-        | Screenshot | filename=page.png        |                                    # page screenshot, defined filename
-        | Screenshot | filename=${NONE}         |                                    # page screenshot, NO file will be created
-        | Screenshot |                          |                                    # page screenshot, default filename
-        | Screenshot | locator=//img[@alt="Google"] |                                    # element screenshot, default filename
-        | Screenshot | locator=//img[@alt="Google"] | filename=${CURDIR}/subdir/loc.png  # element screenshot, create dirs if not existing
+        | Screenshot | locator=//img[@alt="Google"] | filename=locator.png              | # element screenshot, defined filename            |
+        | Screenshot | filename=page.png            |                                   | # page screenshot, defined filename               |
+        | Screenshot | filename=${NONE}             |                                   | # page screenshot, NO file will be created        |
+        | Screenshot |                              |                                   | # page screenshot, default filename               |
+        | Screenshot | locator=//img[@alt="Google"] |                                   | # element screenshot, default filename            |
+        | Screenshot | locator=//img[@alt="Google"] | filename=${CURDIR}/subdir/loc.png | # element screenshot, create dirs if not existing |
         """  # noqa: E501
         screenshot_keywords = ScreenshotKeywords(self)
         default_filename_prefix = f"screenshot-{int(time.time())}"
@@ -1537,11 +1661,14 @@ class Browser(SeleniumLibrary):
             runnable_keyword(*args, **kwargs)
             return True
         except catches as e:
-            BuiltIn().log(
-                "Ran with keyword <b>%s</b> which returned error: <i>%s</i>"
-                % (runnable_keyword.__func__.__name__.replace("_", " ").title(), e),
-                html=True,
-            )
+            try:
+                BuiltIn().log(
+                    "Ran with keyword <b>%s</b> which returned error: <i>%s</i>"
+                    % (runnable_keyword.__func__.__name__.replace("_", " ").title(), e),
+                    html=True,
+                )
+            except RobotNotRunningError:
+                pass
             return False
 
     @keyword
@@ -1638,7 +1765,7 @@ class Browser(SeleniumLibrary):
         """
         Highlight all matching elements by locator.
 
-        Highlighing is done by adding a colored outline
+        Highlighting is done by adding a colored outline
         around the elements with CSS styling.
 
         ``locator``  element locator
@@ -1689,3 +1816,88 @@ class Browser(SeleniumLibrary):
             for idx in range(len(elements))
         )
         self.driver.execute_script(script, *elements)
+
+    @keyword
+    def print_to_pdf(self, output_path: str = None, params: dict = None):
+        """
+        Print the current page to a PDF document using Chromium devtools.
+
+        For supported parameters see:
+        https://chromedevtools.github.io/devtools-protocol/tot/Page/#method-printToPDF
+
+        ``output_path`` filepath for the generated pdf. By default it is saved to
+          the output folder with name `out.pdf`.
+
+        ``params`` parameters for the Chrome print method. By default uses values:
+
+        ``{
+            "landscape": False,
+            "displayHeaderFooter": False,
+            "printBackground": True,
+            "preferCSSPageSize": True,
+        }``
+        """
+        if "chrom" not in self.driver.name:
+            raise NotImplementedError("PDF printing works only with Chrome/Chromium")
+
+        default_params = {
+            "landscape": False,
+            "displayHeaderFooter": False,
+            "printBackground": True,
+            "preferCSSPageSize": True,
+        }
+
+        try:
+            output_dir = BuiltIn().get_variable_value("${OUTPUT_DIR}", "output")
+        except robot.libraries.BuiltIn.RobotNotRunningError:
+            output_dir = "output"
+        default_output = f"{output_dir}/out.pdf"
+        output_path = output_path or default_output
+
+        params = params or default_params
+        result = self._send_command_and_get_result("Page.printToPDF", params)
+        pdf = base64.b64decode(result["data"])
+
+        with open(output_path, "wb") as f:
+            f.write(pdf)
+
+        return output_path
+
+    @keyword
+    def execute_cdp(self, command, parameters):
+        """
+        Executes Chrome DevTools Protocol commands
+
+        Works only with Chrome/Chromium
+
+        For more information, available commands and parameters, see:
+        https://chromedevtools.github.io/devtools-protocol/
+
+        ``command`` command to execute as string
+
+        ``parameters`` parameters for command as a dictionary
+
+        Example:
+
+        | Open Chrome Browser | about:blank | headless=True |
+        | &{params} | Create Dictionary | useragent=Chrome/83.0.4103.53 |
+        | Execute CDP | Network.setUserAgentOverride | ${params} |
+        | Go To | https://robocorp.com |
+        """
+        if "chrom" not in self.driver.name:
+            raise NotImplementedError(
+                "Executing Chrome DevTools Protocol commands "
+                "works only with Chrome/Chromium"
+            )
+        return self._send_command_and_get_result(command, parameters)
+
+    def _send_command_and_get_result(self, cmd, params):
+        resource = (
+            f"session/{self.driver.session_id}/chromium/send_command_and_get_result"
+        )
+        # pylint: disable=protected-access
+        url = f"{self.driver.command_executor._url}/{resource}"
+        body = json.dumps({"cmd": cmd, "params": params})
+        response = self.driver.command_executor._request("POST", url, body)
+
+        return response.get("value")
